/* jshint globalstrict:true */
'use strict';

const DESIGNS_CONTAINER_ID = 'locrmaps-overlay-templates';

let printFormat;
let overlayTemplateBaseColor = 'white';
let overlayTemplateTextColor = 'black';

document.addEventListener('DOMContentLoaded', function() {
    let overlayTemplatesElement;

    const locrmapsOverlayTemplatesQuerySelector = document.getElementById('locrmaps-overlay-templates-query-selector');
    if (locrmapsOverlayTemplatesQuerySelector instanceof HTMLInputElement && locrmapsOverlayTemplatesQuerySelector.value) {
        overlayTemplatesElement = document.querySelector(locrmapsOverlayTemplatesQuerySelector.value);
    }
    if (!overlayTemplatesElement) {
        overlayTemplatesElement = document.getElementById(DESIGNS_CONTAINER_ID);
    }

    if (overlayTemplatesElement instanceof HTMLSelectElement) {
        overlayTemplatesElement.addEventListener('change', function(event) {
            if (!(event.target instanceof HTMLSelectElement)) {
                return;
            }

            document.dispatchEvent(new CustomEvent('locr.locrmaps.set-overlay-template', { detail: { overlayTemplateData: buildMapOverlayFromTemplateName(event.target.value) } }));
        });

        if (overlayTemplatesElement.value) {
            document.dispatchEvent(new CustomEvent('locr.locrmaps.set-overlay-template', { detail: { overlayTemplateData: buildMapOverlayFromTemplateName(overlayTemplatesElement.value) } }));

            wp?.hooks?.addAction(
                'locrMAPSInitialized',
                'locr/locrmaps',
                () => document.dispatchEvent(new CustomEvent('locr.locrmaps.set-overlay-template', { detail: { overlayTemplateData: buildMapOverlayFromTemplateName(overlayTemplatesElement.value) } }))
            );
        }

        if (wp?.hooks) {
            wp.hooks.addAction(
                'locrMAPSSetPrintFormat',
                'locr/locrmaps',
                data => {
                    printFormat = data.printFormat;
                    document.dispatchEvent(new CustomEvent('locr.locrmaps.set-overlay-template', { detail: { overlayTemplateData: buildMapOverlayFromTemplateName(overlayTemplatesElement.value) } }));
                }
            );
    
            wp.hooks.addAction(
                'locrMAPSForWooCommerceSetOverlayTemplateBaseColor',
                'locr/locrmaps',
                data => {
                    overlayTemplateBaseColor = data.value;
                    document.dispatchEvent(new CustomEvent('locr.locrmaps.set-overlay-template', { detail: { overlayTemplateData: buildMapOverlayFromTemplateName(overlayTemplatesElement.value) } }));
                }
            );
    
            wp.hooks.addAction(
                'locrMAPSForWooCommerceSetOverlayTemplateTextColor',
                'locr/locrmaps',
                data => {
                    overlayTemplateTextColor = data.value;
                    document.dispatchEvent(new CustomEvent('locr.locrmaps.set-overlay-template', { detail: { overlayTemplateData: buildMapOverlayFromTemplateName(overlayTemplatesElement.value) } }));
                }
            );
        }
    }
});

/**
 * @param {string} svg
 * @returns {*}
 */
function buildMapOverlayFromTemplateName(design) {
    return {
        fontFaces: [
            {
                family: 'locr-copyright',
                source: 'https://fonts.gstatic.com/s/barlowcondensed/v12/HTxwL3I-JCGChYJ8VI-L6OO_au7B4-Lwz3bWuQ.woff2',
                descriptors: {
                    style: 'normal',
                    weight: '500',
                    unicodeRange: 'U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+0304, U+0308, U+0329, U+2000-206F, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD'
                }
            },
            {
                family: 'Barlow Condensed',
                source: 'https://fonts.gstatic.com/s/barlowcondensed/v12/HTxwL3I-JCGChYJ8VI-L6OO_au7B47rxz3nWuZEC.woff2',
                descriptors: {
                    style: 'normal',
                    weight: '300',
                    unicodeRange: 'U+0102-0103, U+0110-0111, U+0128-0129, U+0168-0169, U+01A0-01A1, U+01AF-01B0, U+0300-0301, U+0303-0304, U+0308-0309, U+0323, U+0329, U+1EA0-1EF9, U+20AB'
                }
            },
            {
                family: 'Barlow Condensed',
                source: 'https://fonts.gstatic.com/s/barlowcondensed/v12/HTxwL3I-JCGChYJ8VI-L6OO_au7B47rxz3jWuZEC.woff2',
                descriptors: {
                    style: 'normal',
                    weight: '300',
                    unicodeRange: 'U+0100-02BA, U+02BD-02C5, U+02C7-02CC, U+02CE-02D7, U+02DD-02FF, U+0304, U+0308, U+0329, U+1D00-1DBF, U+1E00-1E9F, U+1EF2-1EFF, U+2020, U+20A0-20AB, U+20AD-20C0, U+2113, U+2C60-2C7F, U+A720-A7FF'
                }
            },
            {
                family: 'Barlow Condensed',
                source: 'https://fonts.gstatic.com/s/barlowcondensed/v12/HTxwL3I-JCGChYJ8VI-L6OO_au7B47rxz3bWuQ.woff2',
                descriptors: {
                    style: 'normal',
                    weight: '300',
                    unicodeRange: 'U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+0304, U+0308, U+0329, U+2000-206F, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD'
                }
            },
            {
                family: 'Barlow Condensed',
                source: 'https://fonts.gstatic.com/s/barlowcondensed/v12/HTxwL3I-JCGChYJ8VI-L6OO_au7B4-Lwz3nWuZEC.woff2',
                descriptors: {
                    style: 'normal',
                    weight: '500',
                    unicodeRange: 'U+0102-0103, U+0110-0111, U+0128-0129, U+0168-0169, U+01A0-01A1, U+01AF-01B0, U+0300-0301, U+0303-0304, U+0308-0309, U+0323, U+0329, U+1EA0-1EF9, U+20AB'
                }
            },
            {
                family: 'Barlow Condensed',
                source: 'https://fonts.gstatic.com/s/barlowcondensed/v12/HTxwL3I-JCGChYJ8VI-L6OO_au7B4-Lwz3jWuZEC.woff2',
                descriptors: {
                    style: 'normal',
                    weight: '500',
                    unicodeRange: 'U+0100-02BA, U+02BD-02C5, U+02C7-02CC, U+02CE-02D7, U+02DD-02FF, U+0304, U+0308, U+0329, U+1D00-1DBF, U+1E00-1E9F, U+1EF2-1EFF, U+2020, U+20A0-20AB, U+20AD-20C0, U+2113, U+2C60-2C7F, U+A720-A7FF'
                }
            },
            {
                family: 'Barlow Condensed',
                source: 'https://fonts.gstatic.com/s/barlowcondensed/v12/HTxwL3I-JCGChYJ8VI-L6OO_au7B4-Lwz3bWuQ.woff2',
                descriptors: {
                    style: 'normal',
                    weight: '500',
                    unicodeRange: 'U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+0304, U+0308, U+0329, U+2000-206F, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD'
                }
            }
        ],
        layers: [
            {
                id: 'mask',
                type: 'svg',
                content: buildSvgMaskFromTemplateName(design)
            },
            {
                id: 'headline',
                type: 'text',
                content: buildTextOverlay('headline'),
                attributes: buildTextOverlayAttributes('headline', design)
            },
            {
                id: 'subline',
                type: 'text',
                content: buildTextOverlay('subline'),
                attributes: buildTextOverlayAttributes('subline', design)
            },
            {
                id: 'tagline',
                type: 'text',
                content: buildTextOverlay('tagline'),
                attributes: buildTextOverlayAttributes('tagline', design)
            }
        ]
    };
}

/**
 * @param {string} design
 * @param {number} y
 * @returns {number}
 */
function adjustTextYOffsetForDesign(design, y) {
    switch (design) {
        case 'clean':
            break;
        default:
            y -= 12;
            break;
    }

    return y;
}

/**
 * @param {string} design
 * @returns {string}
 */
function buildSvgMaskFromTemplateName(design) {
    const copyrightText = '© LOCR/OpenStreetMap';
    const copyrightFontSize = '32px';
    const copyrightStrokeWidth = '8px';

    let copyrightViewBox = '0 0 200 240';
    if (printFormat) {
        const printFormatSize = printFormat.getSize();
        copyrightViewBox = `0 0 ${printFormatSize.width} ${printFormatSize.height}`;
    }

    let copyrightX = 0;
    let copyrightY = 0;

    switch (design) {
        case 'circle':
            let maskViewBox = '0 0 200 240';
            let cx = 100;
            let cy = 100;
            let r = 100 * 0.8;
            if (printFormat) {
                const printFormatSize = printFormat.getSize();
                const printFormatSizeRatio = printFormatSize.width / printFormatSize.height;

                maskViewBox = `0 0 ${printFormatSize.width} ${printFormatSize.height}`;

                cx = printFormatSize.width / 2;
                cy = printFormatSize.height / 2 - printFormatSize.height / 2 * 0.166666667;

                if (printFormatSizeRatio >= 1) { // landscape
                    r = printFormatSize.height / 2 * 0.6;
                } else { // portrait
                    r = printFormatSize.width / 2 * 0.8;
                }

                copyrightX = printFormatSize.width / 2 + r - 24;
                copyrightY = printFormatSize.height / 2 - printFormatSize.height / 2 * 0.166666667;
            }

            return `<g>
                <defs>
                    <mask id="${design}-mask">
                        <rect width="100%" height="100%" fill="white"/>
                        <svg width="100%" height="100%" viewBox="${maskViewBox}" preserveAspectRatio="xMidYMid meet">
                            <circle fill="black" cx="${cx}" cy="${cy}" r="${r}"/>
                        </svg>
                    </mask>
                    <mask id="${design}-border-mask">
                        <rect width="100%" height="100%" fill="black"/>
                        <svg width="100%" height="100%" viewBox="${maskViewBox}" preserveAspectRatio="xMidYMid meet">
                            <circle fill="black" cx="${cx}" cy="${cy}" r="${r}" stroke="white" stroke-width="5"/>
                        </svg>
                    </mask>
                    <mask id="${design}-copyright-outline-mask">
                        <svg width="100%" height="100%" viewBox="${copyrightViewBox}" preserveAspectRatio="xMidYMid meet">
                            <text id="copyright" text-anchor="middle" x="${copyrightX}" y="${copyrightY}" transform="rotate(-90 ${copyrightX} ${copyrightY})" fill="white" font-family="locr-copyright" font-size="${copyrightFontSize}" stroke="white" stroke-width="${copyrightStrokeWidth}">${copyrightText}</text>
                        </svg>
                    </mask>
                    <mask id="${design}-copyright-mask">
                        <svg width="100%" height="100%" viewBox="${copyrightViewBox}" preserveAspectRatio="xMidYMid meet">
                            <text id="copyright" text-anchor="middle" x="${copyrightX}" y="${copyrightY}" transform="rotate(-90 ${copyrightX} ${copyrightY})" fill="white" font-family="locr-copyright" font-size="${copyrightFontSize}">${copyrightText}</text>
                        </svg>
                    </mask>
                </defs>
                <rect width="100%" height="100%" fill="${overlayTemplateBaseColor}" mask="url(#${design}-mask)"/>
                <rect width="100%" height="100%" fill="black" mask="url(#${design}-border-mask)"/>
                <rect width="100%" height="100%" fill="white" mask="url(#${design}-copyright-outline-mask)"/>
                <rect width="100%" height="100%" fill="black" mask="url(#${design}-copyright-mask)"/>
            </g>`;

        case 'clean':
            if (printFormat) {
                const printFormatSize = printFormat.getSize();
                copyrightX = printFormatSize.width - 12;
                copyrightY = printFormatSize.height * 0.86 - 16;
            }

            return `<g>
                <defs>
                    <mask id="${design}-mask">
                        <rect width="100%" height="100%" fill="white"/>
                        <rect width="100%" height="86%" fill="black"/>
                    </mask>
                    <mask id="${design}-copyright-outline-mask">
                        <svg width="100%" height="100%" viewBox="${copyrightViewBox}" preserveAspectRatio="xMidYMid meet">
                            <text id="copyright" text-anchor="end" x="${copyrightX}" y="${copyrightY}" fill="white" font-family="locr-copyright" font-size="${copyrightFontSize}" stroke="white" stroke-width="${copyrightStrokeWidth}">${copyrightText}</text>
                        </svg>
                    </mask>
                    <mask id="${design}-copyright-mask">
                        <svg width="100%" height="100%" viewBox="${copyrightViewBox}" preserveAspectRatio="xMidYMid meet">
                            <text id="copyright" text-anchor="end" x="${copyrightX}" y="${copyrightY}" fill="white" font-family="locr-copyright" font-size="${copyrightFontSize}">${copyrightText}</text>
                        </svg>
                    </mask>
                </defs>
                <rect width="100%" height="100%" fill="${overlayTemplateBaseColor}" mask="url(#${design}-mask)"/>
                <rect width="100%" height="100%" fill="white" mask="url(#${design}-copyright-outline-mask)"/>
                <rect width="100%" height="100%" fill="black" mask="url(#${design}-copyright-mask)"/>
            </g>`;  

        case 'heart':
            const maskViewBoxDataHeart = {
                x: 0,
                y: 0,
                width: 200,
                height: 240
            };
            if (printFormat) {
                const printFormatSize = printFormat.getSize();
                const printFormatSizeRatio = printFormatSize.width / printFormatSize.height;
                const viewBoxSizeRatio = maskViewBoxDataHeart.width / maskViewBoxDataHeart.height;

                let viewBoxYOffset = 0;
                let scaledViewBoxSize = {
                    width: printFormatSize.height * viewBoxSizeRatio,
                    height: printFormatSize.height
                };

                if (viewBoxSizeRatio >= printFormatSizeRatio) {
                    scaledViewBoxSize = {
                        width: printFormatSize.width,
                        height: printFormatSize.width / viewBoxSizeRatio
                    };
                    viewBoxYOffset = (printFormatSize.height - scaledViewBoxSize.height) / 2;
                }

                copyrightX = printFormatSize.width / 2;
                copyrightY = viewBoxYOffset + scaledViewBoxSize.height * 0.7425;
            }

            return `<g>
                <defs>
                    <mask id="${design}-mask">
                        <rect width="100%" height="100%" fill="white"/>
                        <svg width="100%" height="100%" viewBox="0 0 200 240" preserveAspectRatio="xMidYMid meet">
                            <path fill="black" transform="translate(100, 100) scale(0.8) translate(-100, -140)" d="M200,116.8c0-34.6-25.5-62.6-57-62.6s-32.6,8.3-43,21.5c-10.4-13.2-25.8-21.5-43-21.5-31.5,0-57,28-57,62.6-.2,17.6,6.8,34.6,19.3,47l80.7,76.2,80.7-76.2c12.5-12.4,19.5-29.3,19.3-47h0Z"/>
                        </svg>
                    </mask>
                    <mask id="${design}-copyright-outline-mask">
                        <svg width="100%" height="100%" viewBox="${copyrightViewBox}" preserveAspectRatio="xMidYMid meet">
                            <text id="copyright" text-anchor="start" x="${copyrightX}" y="${copyrightY}" transform="rotate(-43 ${copyrightX} ${copyrightY})" fill="white" font-family="locr-copyright" font-size="${copyrightFontSize}" stroke="white" stroke-width="${copyrightStrokeWidth}">${copyrightText}</text>
                        </svg>
                    </mask>
                    <mask id="${design}-copyright-mask">
                        <svg width="100%" height="100%" viewBox="${copyrightViewBox}" preserveAspectRatio="xMidYMid meet">
                            <text id="copyright" text-anchor="start" x="${copyrightX}" y="${copyrightY}" transform="rotate(-43 ${copyrightX} ${copyrightY})" fill="white" font-family="locr-copyright" font-size="${copyrightFontSize}">${copyrightText}</text>
                        </svg>
                    </mask>
                </defs>
                <rect width="100%" height="100%" fill="${overlayTemplateBaseColor}" mask="url(#${design}-mask)"/>
                <rect width="100%" height="100%" fill="white" mask="url(#${design}-copyright-outline-mask)"/>
                <rect width="100%" height="100%" fill="black" mask="url(#${design}-copyright-mask)"/>
            </g>`;

        case 'pin':
            const maskViewBoxDataPin = {
                x: 0,
                y: 0,
                width: 200,
                height: 240
            };
            if (printFormat) {
                const printFormatSize = printFormat.getSize();
                const printFormatSizeRatio = printFormatSize.width / printFormatSize.height;
                const viewBoxSizeRatio = maskViewBoxDataPin.width / maskViewBoxDataPin.height;

                let viewBoxYOffset = 0;
                let scaledViewBoxSize = {
                    width: printFormatSize.height * viewBoxSizeRatio,
                    height: printFormatSize.height
                };

                if (viewBoxSizeRatio >= printFormatSizeRatio) {
                    scaledViewBoxSize = {
                        width: printFormatSize.width,
                        height: printFormatSize.width / viewBoxSizeRatio
                    };
                    viewBoxYOffset = (printFormatSize.height - scaledViewBoxSize.height) / 2;
                }

                copyrightX = printFormatSize.width / 2;
                copyrightY = viewBoxYOffset + scaledViewBoxSize.height * 0.742;
            }

            return `<g>
                <defs>
                    <mask id="${design}-mask">
                        <rect width="100%" height="100%" fill="white"/>
                        <svg width="100%" height="100%" viewBox="0 0 200 240" preserveAspectRatio="xMidYMid meet">
                            <path transform="translate(100, 100) scale(0.8) translate(-100, -140)" fill="black" d="M147.6,56.8c-32.7-26.3-80.5-21.2-106.8,11.5-22.2,27.6-22.4,66.8-.5,94.6l59.7,77.1,59.1-76.4c26.3-32.7,21.2-80.5-11.5-106.8h0Z"/>
                        </svg>
                    </mask>
                    <mask id="${design}-copyright-outline-mask">
                        <svg width="100%" height="100%" viewBox="${copyrightViewBox}" preserveAspectRatio="xMidYMid meet">
                            <text id="copyright" text-anchor="start" x="${copyrightX}" y="${copyrightY}" transform="rotate(-52.5 ${copyrightX} ${copyrightY})" fill="white" font-family="locr-copyright" font-size="${copyrightFontSize}" stroke="white" stroke-width="${copyrightStrokeWidth}">${copyrightText}</text>
                        </svg>
                    </mask>
                    <mask id="${design}-copyright-mask">
                        <svg width="100%" height="100%" viewBox="${copyrightViewBox}" preserveAspectRatio="xMidYMid meet">
                            <text id="copyright" text-anchor="start" x="${copyrightX}" y="${copyrightY}" transform="rotate(-52.5 ${copyrightX} ${copyrightY})" fill="white" font-family="locr-copyright" font-size="${copyrightFontSize}">${copyrightText}</text>
                        </svg>
                    </mask>
                </defs>
                <rect width="100%" height="100%" fill="${overlayTemplateBaseColor}" mask="url(#${design}-mask)"/>
                <rect width="100%" height="100%" fill="white" mask="url(#${design}-copyright-outline-mask)"/>
                <rect width="100%" height="100%" fill="black" mask="url(#${design}-copyright-mask)"/>
            </g>`;

        default:
            return `<g>
                <defs>
                    <mask id="${design}-mask">
                        <rect width="100%" height="100%" fill="white"/>
                        <circle cx="50%" cy="50%" r="30%" fill="red"/>
                    </mask>
                </defs>
                <rect width="100%" height="100%" fill="red" opacity="0.5" mask="url(#${design}-mask)"/>
            </g>`;
    }
}

/**
 * @param {string} overlayRefId
 * @returns {string}
 */
function buildTextOverlay(overlayRefId) {
    const locrmapsTextOverlayElements = document.getElementsByName('locrmaps-text-overlay');
    for (const locrmapsTextOverlayElement of locrmapsTextOverlayElements) {
        if ((locrmapsTextOverlayElement instanceof HTMLInputElement || locrmapsTextOverlayElement instanceof HTMLTextAreaElement) && locrmapsTextOverlayElement.getAttribute('data-overlay-ref-id') === overlayRefId) {
            return locrmapsTextOverlayElement.value;
        }
    }

    return '';
}

/**
 * @param {string} design
 * @returns {Object}
 */
function buildHeadlineTextOverlayAttributes(design) {
    const fontHeightFactor = calculateCurrentFontHeightFactorFromBaseHeight();

    return {
        x: '50%',
        y: `${adjustTextYOffsetForDesign(design, 91)}%`,
        fill: overlayTemplateTextColor,
        'font-size': `${convertFrom90DpiTo300Dpi(12) * fontHeightFactor}mm`
    };
}

/**
 * @param {string} design
 * @returns {Object}
 */
function buildSublineTextOverlayAttributes(design) {
    const fontHeightFactor = calculateCurrentFontHeightFactorFromBaseHeight();

    return {
        x: '50%',
        y: `${adjustTextYOffsetForDesign(design, 94.5)}%`,
        fill: overlayTemplateTextColor,
        'font-size': `${convertFrom90DpiTo300Dpi(5) * fontHeightFactor}mm`,
        'font-weight': '300'
    };
}

/**
 * @param {string} design
 * @returns {Object}
 */
function buildTaglineTextOverlayAttributes(design) {
    const fontHeightFactor = calculateCurrentFontHeightFactorFromBaseHeight();

    return {
        x: '50%',
        y: `${adjustTextYOffsetForDesign(design, 97)}%`,
        fill: overlayTemplateTextColor,
        'font-size': `${convertFrom90DpiTo300Dpi(5) * fontHeightFactor}mm`,
        'font-weight': '300'
    };
}

/**
 * @param {string} overlayRefId
 * @param {string} design
 * @returns {Object}
 */
function buildTextOverlayAttributes(overlayRefId, design) {
    switch (overlayRefId) {
        case 'headline':
            return buildHeadlineTextOverlayAttributes(design);
        case 'subline':
            return buildSublineTextOverlayAttributes(design);
        case 'tagline':
            return buildTaglineTextOverlayAttributes(design);
        default:
            return {};
    }
}

/**
 * @param {number} value
 * @returns {number}
 */
function convertFrom90DpiTo300Dpi(value) {
    return value * 300 / 90;
}

function calculateCurrentFontHeightFactorFromBaseHeight(baseHeight = 30) {
    let fontHeightFactor = 1;

    if (printFormat) {
        const printFormatSize = printFormat.getSize();
        const cmHeight = printFormatSize.height / 300 * 2.54;
        fontHeightFactor = cmHeight / baseHeight;
    }

    return fontHeightFactor;
}
